#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <vector>
#include <list>
#include <math.h>
#include <stdlib.h>
#include <algorithm> 

#include "libgam/libgam.h"
#include "cgars.h"

using namespace std;

//##################
//## mainCLS defs ##
//##################



string usage_analyze = "      SYNOPSIS \n \t cgars analyze [options] <name for output files>\n\n \
      DESCRIPTION\n \
      \t -h -? -help \t help\n \
      \t -cn         \t filename of raw copy number data\n \
      \t -i          \t name of sample information file\n \
      \t -m          \t rank matrix file\n \
      \t -s          \t name of segment file \n \
      \t -b          \t genome build (hg18, hg17, mm9) [hg18]\n \
      \t -nocnv      \t represses removal of CNV prone probes\n \
      \t -l          \t file to exclude regions form analysis\n \
      \t -mwd        \t minimal number of probes for a segment [20]\n \
      \t -alpha      \t smoothing parameter for segmentation [0.00001]\n \
      \t -uq         \t upper quantile for amplifications (list, e.g, 0.15,0.1,...) [0.15]\n \
      \t -lq         \t lower quantile for deletions (list, e.g, 0.15,0.1,...) [0.15]\n \
      \t -alfq       \t q-value cutoff SCNA filter [0.25]\n \
      \t -keepXY     \t keep X and Y chromosomes\n";


string usage_plot = "      SYNOPSIS \n \t cgars plot [options] <prefix alf-file (prefix\"_alf.txt\")>\n\n \
      DESCRIPTION\n \
      \t -h -? -help \t help\n \
      \t -a          \t significance threshold [0.05]\n \
      \t -p          \t peel-off filter  [2]\n \
      \t -b          \t genome build (hg18 or hg17) [hg18]\n";


//####################
//## analyze module ##
//####################


void mainCLS::parse_analyze(int argc,char *argv[])
{
  int longindex,opt,i;
  string err,tmp,cmdline="";
  stringstream line;
  double dtmp;
  
  //store comand line
  for(i=0;i<argc;i++)
    {
      tmp=argv[i];
      cmdline+=tmp+" ";
    }
  //default parameters/initialize
  cnName="";
  infoName="";
  mwd=20;
  build="hg18";
  alpha=0.00001;
  alfq=0.25;
  dataFormat="cn"; //snp format is obsolete
  exclList="";

  isCN=0; isInfo=0; isMat=0; isSeg=0; isXY=0;
  nocnv=0;
  // option definition
  static struct option longopts[]={
    {"help"  , 0, 0,  'h'},
    {"help"  , 0, 0,  '?'},
    {"cn"    , 1, 0,    1},
    {"i"     , 1, 0,    2},
    {"uq"    , 1, 0,    3},
    {"lq"    , 1, 0,    4},
    {"s"     , 1, 0,    5},
    {"mwd"   , 1, 0,    7},
    {"alpha" , 1, 0,    8},
    {"m"     , 1, 0,    9},
    {"b"     , 1, 0,   11},
    {"nocnv" , 0, 0,   12},
    {"l"     , 1, 0,   13},
    {"keepXY", 0, 0,   17},
    {"alfq"  , 1, 0,   19},
    {0, 0, 0, 0}
  }; 

  optind=0;
  //parse command line arguments
  while((opt=getopt_long_only(argc,argv,"h?",longopts,&longindex)) != -1)
    {
      switch(opt)
	{
	case 'h':
	  print_header();
	  cerr << usage_analyze << endl;
	  exit(1);
	  break;
	case '?':
	  print_header();
	  cerr << usage_analyze << endl;
	  exit(1);	
	  break;
	case 1:
	  cnName=(string)optarg;
	  isCN=1;
	  break;
	case 2:
	  infoName=(string)optarg;
	  isInfo=1;
	  break;
	case 3:
	  tmp = (string)optarg+",";
	  line.str(""); line.clear();
	  line << tmp;
	  while(getline(line,tmp,','))
	    upperq.push_back(atof(tmp.c_str()));
	  break;
	case 4:
	  tmp = (string)optarg+",";
	  line.str(""); line.clear();
	  line << tmp;
	  while(getline(line,tmp,','))
	    lowerq.push_back(atof(tmp.c_str()));
	  break;
	case 5:
	  segName=(string)optarg;
	  isSeg=1;
	  break;
	case 7:
	  mwd=atol(optarg);
	  break;
	case 8:
	  dtmp=atof(optarg);
	  if(dtmp < 0 || dtmp > 1 )
	      {
	      err="Error: alpha must between 0 and 1.";
	      throw err;
	    }
	  else
	    alpha=dtmp;
	  break;
	case 9:
	  matName=(string)optarg;
	  isMat=1;
	  break;
	case 11:
	  build=(string)optarg;
	  if(build!="hg17" && build!="hg18" && build!="mm9")
	    {
	      err="Error: build must either be hg18 or hg17.";
	      throw err;
	    }
	  break;
	case 12:
	  nocnv=1;
	  break;
	case 13:
	  exclList=(string)optarg;
	  break;
	case 17:
	  isXY=1;
	  break;
	case 19:
	  alfq=atof(optarg);
	  break;
	default:
	  err="Error: cannot parse arguments.";
	  throw err;
	}
    }

  if((argc-optind)!=2)
    {
      print_header();
      cerr << usage_analyze << endl;
      err="Error: please specify a name for the output files.";
      throw err;
    }
  else
    {
      outName=(string)argv[optind+1];
    }

  if(upperq.size()==0)
    upperq.push_back(0.15);

  if(lowerq.size()==0)
    lowerq.push_back(0.15);
  
  tmp=outName+".log";
  logSt.open(tmp.c_str());
  logSt << cmdline << endl;
}

void mainCLS::readRawCN()
{
  vector<long> cnvChr,cnvStart,cnvEnd;
  double tmpd;
  string err,tmp,tmpChr,tmpSNP;
  ifstream in,inExp;
  stringstream line;
  long i,j,ltmp,tmpPos;
  bool isNA;
  
  //read cnv information 
  if(!nocnv)
    {
      
      tmp=(string)INSTALLDIR;
      if(build=="hg17")
	tmp+="/annotation/cnv_hg17.txt";
      if(build=="hg18")
	tmp+="/annotation/cnv_hg18.txt";
      if(build=="mm9")
	tmp+="/annotation/cnv_mm9.txt";

      in.open(tmp.c_str());
      if(in.is_open())
	{
	  while(in >>ltmp)
	    {
	      cnvChr.push_back(ltmp);
	      in >> ltmp;
	      cnvStart.push_back(ltmp);
	      in >> ltmp;
	      cnvEnd.push_back(ltmp);
	    }
	  in.close();
	}
      else
	{
	  cerr << "Warning: cannot find cnv-file: "<< tmp <<"\n";
	  nocnv=1; //if cnv file cannot be found
	}
    }

  if(exclList!="")
    {
      in.open(exclList.c_str());
      if(in.is_open())
	{
	  nocnv=0;
	  while(in >>ltmp)
	    {
	      cnvChr.push_back(ltmp);
	      in >> ltmp;
	      cnvStart.push_back(ltmp);
	      in >> ltmp;
	      cnvEnd.push_back(ltmp);
	    }
	  in.close();
	}
      else
	{
	  err = "Error: cannot open exclude file.";
	  throw err;
	}
    }

  sort_cnv(cnvChr,cnvStart,cnvEnd);

  if(isCN)
    {
      in.open(cnName.c_str());
      if(!in.is_open())
	{
	  err="Error: cannot open "+cnName+".";
	  throw err;
	}
      line.str(""); line.clear();
      getline(in,tmp,'\n');
      line << tmp;
      getline(line,tmp,'\t');
      //check if header contains SNP, Chromosome, PhysicalPosition
      if(tmp!="SNP")
	{
	  err="Error: "+cnName+" has wrong format.";
	  throw err;
	}
      getline(line,tmp,'\t');
      if(tmp!="Chromosome")
	{
	  err="Error: "+cnName+" has wrong format.";
	  throw err;
	}
      getline(line,tmp,'\t');
      if(tmp!="PhysicalPosition")
	{
	  err="Error: "+cnName+" has wrong format.";
	  throw err;
	}
      //read array names and store in arrayNames
      while(!line.eof())
	{
	   getline(line,tmp,'\t');
	   arrayNames.push_back(tmp);
	   if(dataFormat=="snp")
	     getline(line,tmp,'\t');
	}
      nSamp=arrayNames.size();
      sampleNames=arrayNames;
      
      //read the data matrix and write into rawData
      vector<double> tmpData(nSamp,-1e99);
      while(in >> tmpSNP >> tmpChr >> tmpPos)
	{
	  isNA=0;
	  for(j=0;j<nSamp;j++)
	    {
	      in >> tmp;
	      //check for NA's
	      if(tmp=="NA")
		{
		  tmpData[j]=-1e99;
		  isNA=1;
		}
	      else
		tmpData[j]=atof(tmp.c_str());
	      if(dataFormat=="snp")
		in >> tmp;
	    }
	  if((!(isXY==0 && (tmpChr=="X" || tmpChr=="Y"))) && isNA==0)
	    {
	      if(nocnv)
		{	  
		  probeName.push_back(tmpSNP);
		  chromosome.push_back(tmpChr);
		  position.push_back(tmpPos);
		  rawData.push_back(tmpData);  
		}
	      else
		{
		  //check for CNV
		  if(!isInSeg(cnvChr,cnvStart,cnvEnd,atol(tmpChr.c_str()),tmpPos))
		    {
		      probeName.push_back(tmpSNP);
		      chromosome.push_back(tmpChr);
		      position.push_back(tmpPos);
		      rawData.push_back(tmpData);
		    }
		}
	    }
	}
      in.close();
      nPos=rawData.size();

      if(isInfo)
	{
	  in.open(infoName.c_str());
	  if(!in.is_open())
	    {
	      err="Error: cannot open "+infoName+".";
	      throw err;
	    }
	  line.str(""); line.clear();
	  getline(in,tmp,'\n');
	  line << tmp;
	  getline(line,tmp,'\t');
	  //check if header contains Array, Sample
	  if(tmp!="Array")
	    {
	      err="Error: "+infoName+" has wrong format.";
	      throw err;
	    }
	  getline(line,tmp,'\t');
	  if(tmp!="Sample")
	    {
	      err="Error: "+infoName+" has wrong format.";
	      throw err;
	    }
	  //map sample data
	  while(getline(in,tmp,'\n'))
	    {
	      line.str(""); line.clear();
	      line << tmp;
	      getline(line,tmp,'\t');
	      for(i=0;i<nSamp;i++)
		{
		  if(tmp==arrayNames[i])
		    {
		      getline(line,tmp,'\t');
		      sampleNames[i]=tmp;
		      break;
		    }
		}
	    }
	}
    }
  else if(isMat)
    {
      isInfo=1;
      in.open(matName.c_str());
      if(!in.is_open())
	{
	  err="Error: cannot open "+matName+".";
	  throw err;
	}
      line.str(""); line.clear();
      getline(in,tmp,'\n');
      line << tmp;
      getline(line,tmp,'\t');
      //check if header contains SNP, Chromosome, PhysicalPosition
      if(tmp!="Probe")
	{
	  err="Error: "+matName+" has wrong format.";
	  throw err;
	}
      getline(line,tmp,'\t');
      if(tmp!="Chromosome")
	{
	  err="Error: "+matName+" has wrong format.";
	  throw err;
	}
      getline(line,tmp,'\t');
      if(tmp!="PhysicalPosition")
	{
	  err="Error: "+matName+" has wrong format.";
	  throw err;
	}
      //read array and sample names and store in arrayNames
      while(!line.eof())
	{
	   getline(line,tmp,'\t');
	   arrayNames.push_back(tmp);
	   getline(line,tmp,'\t');
	   sampleNames.push_back(tmp);
	}
      nSamp=arrayNames.size();
      vector<long> tmpData(nSamp,-1);
      nRanks=vector<long>(nSamp,0);
      while(in >> tmp)
	{
	  probeName.push_back(tmp);
	  in >> tmp;
	  chromosome.push_back(tmp);
	  in >> i;
	  position.push_back(i);
	  for(j=0;j<nSamp;j++)
	    {
	      in >> tmp;
	      tmpData[j]=atol(tmp.c_str());
	      if(tmpData[j]!=-1)
		nRanks[j]++;
	    }
	  rankData.push_back(tmpData);
	}
      in.close();
      nPos=rankData.size();
    }
  else
    isInfo=0;
}
void mainCLS::readSeg()
{
  ifstream in;
  string err,tmp;
  long start,end;

  if(isSeg)
    {
      in.open(segName.c_str());

      if(!in.is_open())
	{
	  err="Error: cannot open "+segName+".";
	  throw err;
	}
      
      getline(in,tmp,'\n');
      if(tmp!="Type\tStart\tEnd")
	{
	  err="Error: "+segName+" has wrong format";
	  throw err;
	}
      startSegAmp.clear(); endSegAmp.clear();
      startSegDel.clear(); endSegDel.clear();

      while(in >> tmp >> start >> end)
	{
	  if(tmp=="Amp")
	    {
	      startSegAmp.push_back(start);
	      endSegAmp.push_back(end);
	    }
	  else if(tmp=="Del")
	    {
	      startSegDel.push_back(start);
	      endSegDel.push_back(end);
	    }
	  else
	    {
	      err="Error: "+segName+" has wrong format";
	      throw err;
	    }
	}
      
    }
} 

void mainCLS::computeRanks()
{
  long i,j,nSort;
  ofstream out;
  string err,tmp;
  
  if(isMat)
    return;

  if(!isCN)
    {
      err="Error: raw copy numbers needed to compute ranks.";
      throw err;
    }
  vector<long> indRawDat(nPos+1,0);
  double *sampDat=dvector(1,nPos);
  long *indx=lvector(1,nPos);

  //initialize rank matrix
  rankData=vector<vector<long> >(nPos,vector<long>(nSamp,-1));
  //copy data to nr-type of vector and sort
  for(j=0;j<nSamp;j++)
    {
      nSort=0;
      for(i=0;i<nPos;i++)
	{
	  if(rawData[i][j]!=-1e99)
	    {
	      nSort++;
	      indRawDat[nSort]=i;
	      sampDat[nSort]=rawData[i][j];
	    }
	}
      nRanks.push_back(nSort);
      indexx(nSort,sampDat,indx);
      for(i=1;i<=nSort;i++)
	{
	  rankData[indRawDat[indx[i]]][j]=i;
	}
    }

  //write rank matrix to *.mat file
  tmp=outName+".mat";
  out.open(tmp.c_str());
  
  out << "Probe\tChromosome\tPhysicalPosition";
  for(i=0;i<nSamp;i++)
    out << "\t" << arrayNames[i] << "\t" << sampleNames[i];
  out << endl;
  for(i=0;i<nPos;i++)
    {
      out << probeName[i] << "\t" << chromosome[i] << "\t" << position[i];
      for(j=0;j<nSamp;j++)
	out << "\t" << rankData[i][j];
      out << endl;
    }
  out.close();

  free_dvector(sampDat,1,nPos);
  free_lvector(indx,1,nPos);
}

void mainCLS::computeNullMax(long n)
{
  long i,j,k;
  long np=nRanks[0];
  long idum=-time(NULL);
  long nsamp=1000;
  long nq_Amp = upperq.size();
  long nq_Del = lowerq.size();

  sig_Amp.clear();
  sig_Del.clear();

  double **C_Amp = dmatrix(1,nq_Amp,1,nq_Amp);
  double **C_Del = dmatrix(1,nq_Del,1,nq_Del);
  double res;
  double max,min;

  vector<double> sample_Amp(nq_Amp,0);
  vector<double> sample_Del(nq_Del,0);

  //amplifications
  for(i=0;i<nq_Amp;i++)
    sig_Amp.push_back(sqrt((double)((upperq[i]*np)*(upperq[i]*np)-1)*(upperq[i]*(double)n)/12));
  if(nq_Amp==1) //distribution known no resampling
    {
      sig_max_Amp=1;
      mu_max_Amp=0;
    }
  else
    {
      //create covariance matrix
      for(i=0;i<nq_Amp;i++)
	{
	  for(j=0;j<nq_Amp;j++)
	    {
	      if(sig_Amp[i]/sig_Amp[j] <= 1)
		C_Amp[i+1][j+1] = sig_Amp[i]/sig_Amp[j];
	      else
		C_Amp[i+1][j+1] = sig_Amp[j]/sig_Amp[i];
	    }
	}
      choldc(C_Amp,nq_Amp);
      sig_max_Amp=0; mu_max_Amp=0;
      //resampling
      for(k=0;k<nsamp;k++)
	{
	  max = -1e99;
	  for(i=0;i<nq_Amp;i++)
	    sample_Amp[i] = gasdev(&idum);

	  for(i=0;i<nq_Amp;i++)
	    {
	      res = 0;
	      for(j=0;j<nq_Amp;j++)
		res+=C_Amp[i+1][j+1]*sample_Amp[j];
	      if(res > max)
		max = res;
	    }
	  mu_max_Amp+=max;
	  sig_max_Amp+=max*max;
	}
      mu_max_Amp*=1.0/(double)nsamp;
      sig_max_Amp*=1.0/(double)nsamp;
      sig_max_Amp=sqrt(sig_max_Amp-mu_max_Amp*mu_max_Amp);
    }

  //deletions
  for(i=0;i<nq_Del;i++)
    sig_Del.push_back(sqrt((double)((lowerq[i]*np)*(lowerq[i]*np)-1)*(lowerq[i]*(double)n)/12));
  if(nq_Del==1) //distribution known no resampling
    {
      sig_min_Del=1;
      mu_min_Del=0;
    }
  else
    {
      //create covariance matrix
      for(i=0;i<nq_Del;i++)
	{
	  for(j=0;j<nq_Del;j++)
	    {
	      if(sig_Del[i]/sig_Del[j] <= 1)
		C_Del[i+1][j+1] = sig_Del[i]/sig_Del[j];
	      else
		C_Del[i+1][j+1] = sig_Del[j]/sig_Del[i];
	    }
	}
      choldc(C_Del,nq_Del);
      sig_min_Del=0; mu_min_Del=0;
      //resampling
      for(k=0;k<nsamp;k++)
	{
	  min = 1e99;
	  for(i=0;i<nq_Del;i++)
	    sample_Del[i] = gasdev(&idum);

	  for(i=0;i<nq_Del;i++)
	    {
	      res = 0;
	      for(j=0;j<nq_Del;j++)
		res+=C_Del[i+1][j+1]*sample_Del[j];
	      if(res < min)
		min = res;
	    }
	  mu_min_Del+=min;
	  sig_min_Del+=min*min;
	}
      mu_min_Del*=1.0/(double)nsamp;
      sig_min_Del*=1.0/(double)nsamp;
      sig_min_Del=sqrt(sig_min_Del-mu_min_Del*mu_min_Del);
    }

  free_dmatrix(C_Amp,1,nq_Amp,1,nq_Amp);
  free_dmatrix(C_Del,1,nq_Del,1,nq_Del);

}

void mainCLS::computeRankSum()
{
  long i,j,k;
  long np=nRanks[0];
  double RS,std,max_RS,min_RS;
  long nq_Amp = upperq.size();
  long nq_Del = lowerq.size();
  double cAmp,nAmp;
  double cDel,nDel;
  //initialize rank sums
  rankSumAmp=vector<double>(nPos,0);
  rankSumDel=vector<double>(nPos,0);


  computeNullMax(nSamp);

  //compute rank sum for ampflications
  for(i=0;i<nPos;i++)
    {
      max_RS=-1e99;
      for(k=0;k<nq_Amp;k++)
	{
	  RS=0; std=0;
	  cAmp=(long)((1-upperq[k])*np);
	  nAmp=np-cAmp;
	  for(j=0;j<nSamp;j++)
	    {
	      if(rankData[i][j]>cAmp && rankData[i][j]!=-1)
		{
		  RS+=rankData[i][j]-cAmp-0.5*(nAmp+1.0);
		  std+=(nAmp*nAmp-1.0)/12.0;
		}
	    }
	 	 
	  std=sqrt(std);
	  if(std!=0)
	    RS=RS/std;
	  else
	    RS=0;

	  if(RS > max_RS)
	    max_RS = RS;
	}
      rankSumAmp[i] = (max_RS-mu_max_Amp)/sig_max_Amp;
    }
  
  //compute rank sum for deletions
  for(i=0;i<nPos;i++)
    {
      min_RS=1e99;
      for(k=0;k<nq_Del;k++)
	{
	  RS=0; std=0;
	  cDel=(long)(lowerq[k]*np);
	  nDel=cDel;
	  for(j=0;j<nSamp;j++)
	    {
	      if(rankData[i][j]<=cDel && rankData[i][j]!=-1)
		{
		  RS+=rankData[i][j]-0.5*(nDel+1.0);
		  std+=(nDel*nDel-1.0)/12.0;
		}
	    }

	  std=sqrt(std);
	  if(std!=0)
	    RS=RS/std;
	  else
	    RS=0;

	  if(RS < min_RS)
	    min_RS = RS;
	}
      rankSumDel[i] = (min_RS-mu_min_Del)/sig_min_Del;
    }
}


void mainCLS::segmentRankSum()
{
  long i,j;
  vector<long> startChr,endChr,ind,tmpStart,tmpEnd;
  vector<double> dat;
  string tmp;

  //streams 
  ofstream out;
  ifstream in;

  if(!isSeg)
    {
      //find chromosome boundaries
      startChr.push_back(0);
      for(i=1;i< chromosome.size();i++)
	{
	  if(chromosome[i-1]!=chromosome[i])
	    {
	      endChr.push_back(i-1);
	      startChr.push_back(i);
	    }
	}
      endChr.push_back(chromosome.size()-1);

      for(i=0;i<startChr.size();i++)
	{
	  //##### AMPLIFICATIONS #####
	  ind.clear(); dat.clear();
	  for(j=startChr[i];j<=endChr[i];j++)
	    {
	      ind.push_back(j);
	      dat.push_back(rankSumAmp[j]);
	    }
	  //...and segment it
	  segment(dat,tmpStart,tmpEnd,alpha,mwd);
	  for(j=0;j<tmpStart.size();j++)
	    {
	      startSegAmp.push_back(ind[tmpStart[j]]);
	      endSegAmp.push_back(ind[tmpEnd[j]]);
	    }


	  //##### DELETIONS #####
	  ind.clear(); dat.clear();
	  for(j=startChr[i];j<=endChr[i];j++)
	    {
	      ind.push_back(j);
	      dat.push_back(rankSumDel[j]);
	    }
	  //...and segment it
	  segment(dat,tmpStart,tmpEnd,alpha,mwd);
	  for(j=0;j<tmpStart.size();j++)
	    {
	      startSegDel.push_back(ind[tmpStart[j]]);
	      endSegDel.push_back(ind[tmpEnd[j]]);
	    }
	}
     
      tmp=outName+".seg";
      segName=tmp;
      out.open(tmp.c_str());
      out << "Type\tStart\tEnd\n";
      for(i=0;i<startSegAmp.size();i++)
	out << "Amp\t" << startSegAmp[i] << "\t" << endSegAmp[i] << endl;
      for(i=0;i<startSegDel.size();i++)
	out << "Del\t" << startSegDel[i] << "\t" << endSegDel[i] << endl; 
      out.close();
    }
}

void mainCLS::computeStats()
{
  long i,j,k,h;
  long segStart,segEnd;
  string err;
  double maxL,tmpd,f;
  vector<long> startChr,endChr;
  vector<double> estAcf;


  nSegAmp=startSegAmp.size();
  nSegDel=startSegDel.size();

  if(nSegAmp==0 || nSegDel==0)
    {
      err="Error: noting to compute, check input files.";
      throw err;
    }
  
  //#### Amplifications ####
  maxL=0;
  for(i=0;i<nSegAmp;i++)
    {
      segStart=startSegAmp[i];
      segEnd=endSegAmp[i];
      //what is the maximal length of the segments?
      if(segEnd-segStart+1 > maxL)
	maxL=segEnd-segStart+1;
      tmpd=0;
      for(j=segStart;j<=segEnd;j++)
	tmpd+=rankSumAmp[j];
      meanRSAmp.push_back(tmpd/(double)(segEnd-segStart+1));
    }

  //compute p-values
  acf(rankSumAmp,estAcf,(long)maxL);
  for(j=0;j<nSegAmp;j++)
    {
      h=endSegAmp[j]-startSegAmp[j]+1;
      f=1.0/(double)h;
      for(k=1;k<=h-1;k++)
	f+=2*(double)(h-k)*estAcf[k]/(double)(h*h);
      f=sqrt(f);
      if(meanRSAmp[j]>0)
	pAmp.push_back(erfc(fabs(meanRSAmp[j])/(sqrt(2)*f)));
      else
	pAmp.push_back(1);
    }
  //compute q-values
  qvalues(pAmp,qAmp);

  //#### Deletions ####
  maxL=0;
  for(i=0;i<nSegDel;i++)
    {
      segStart=startSegDel[i];
      segEnd=endSegDel[i];
      //what is the maximal length of the segments?
      if(segEnd-segStart+1 > maxL)
	maxL=segEnd-segStart+1;
      tmpd=0;
      for(j=segStart;j<=segEnd;j++)
	tmpd+=rankSumDel[j];
      meanRSDel.push_back(tmpd/(double)(segEnd-segStart+1));
    }

  //compute p-values
  acf(rankSumDel,estAcf,(long)maxL);
  for(j=0;j<nSegDel;j++)
    {
      h=endSegDel[j]-startSegDel[j]+1;
      f=1.0/(double)h;
      for(k=1;k<=h-1;k++)
	f+=2*(double)(h-k)*estAcf[k]/(double)(h*h);
      f=sqrt(f);
      if(meanRSDel[j]<0)
	pDel.push_back(erfc(fabs(meanRSDel[j])/(sqrt(2)*f)));
      else
	pDel.push_back(1);
    }
  //compute q-values
  qvalues(pDel,qDel);

  annotateSeg();

}

void::mainCLS::leaveOut()
{
  vector <double> RS,std;
  double *wRaw,wPre;
  double max_RS,min_RS;
  long *indx;
  long i,j,k,l,h,kk;
  string tmp,tmp2,err;
  long segStart,segEnd;
  double maxL,f;
  vector<double> estAcf;

  ifstream in;
  
  long np=nRanks[0];
  long nq_Amp=upperq.size();
  long nq_Del=lowerq.size();
  double nAmp;
  double nDel;
  long cAmp;
  long cDel;  

  vector<double> sig_max_Amp_lo,mu_max_Amp_lo;
  vector<double> sig_min_Del_lo,mu_min_Del_lo;
  
  double sig_max_Amp_old,mu_max_Amp_old;
  double sig_min_Del_old,mu_min_Del_old;
  vector<double> sig_Amp_old;
  vector<double> sig_Del_old;

  sig_max_Amp_old=sig_max_Amp; mu_max_Amp_old=mu_max_Amp;
  sig_min_Del_old=sig_min_Del; mu_min_Del_old=mu_min_Del;
  sig_Amp_old=sig_Amp;
  sig_Del_old=sig_Del;

  vector<vector <double> > sig_Amp_lo;
  vector<vector <double> > sig_Del_lo;

  nSegAmp=startSegAmp.size();
  nSegDel=startSegDel.size();
  
  w=vector<double>(nSamp,1);

  //sampling the null distribution
  for(l=1;l<nSamp;l++)
    {
      computeNullMax(l);
      sig_max_Amp_lo.push_back(sig_max_Amp); mu_max_Amp_lo.push_back(mu_max_Amp);
      sig_min_Del_lo.push_back(sig_min_Del); mu_min_Del_lo.push_back(mu_min_Del);
      sig_Amp_lo.push_back(sig_Amp);
      sig_Del_lo.push_back(sig_Del);
    }
  sig_max_Amp_lo.push_back(sig_max_Amp_old); mu_max_Amp_lo.push_back(mu_max_Amp_old);
  sig_min_Del_lo.push_back(sig_min_Del_old); mu_min_Del_lo.push_back(mu_min_Del_old);
  sig_Amp_lo.push_back(sig_Amp_old);
  sig_Del_lo.push_back(sig_Del_old);


  //matrices for leave out statistics
  wRaw=dvector(1,nSamp);
  indx=lvector(1,nSamp);

  vector< vector<double> > lout_RS_Amp(nSamp,vector<double>(startSegAmp.size(),0));
  vector< vector<double> > lout_p_Amp(nSamp,vector<double>(startSegAmp.size(),0));
  vector< vector<double> > lout_q_Amp(nSamp,vector<double>(startSegAmp.size(),0));
  vector< vector<double> > lout_RS_Del(nSamp,vector<double>(startSegDel.size(),0));
  vector< vector<double> > lout_p_Del(nSamp,vector<double>(startSegDel.size(),0));
  vector< vector<double> > lout_q_Del(nSamp,vector<double>(startSegDel.size(),0));

  //compute leave out rank sums, p, and q values for AMPLIFICATIONS
  //1) acf computation for p-value determination
  maxL=0;
  for(i=0;i<nSegAmp;i++)
    {
      segStart=startSegAmp[i];
      segEnd=endSegAmp[i];
      //what is the maximal length of the segments?
      if(segEnd-segStart+1 > maxL)
	maxL=segEnd-segStart+1;
    }

  acf(rankSumAmp,estAcf,(long)maxL);

  //2) segment averaged leave out statistics

  for(kk=0;kk<nSegAmp;kk++)
    {
      for(i=startSegAmp[kk];i<=endSegAmp[kk];i++)
	{
	  RS = vector<double>(nq_Amp,0);
	  std = vector<double>(nq_Amp,0);
	  //index rank matrix
	  for(j=0;j<nSamp;j++)
	    wRaw[j+1] = rankData[i][j];
	  indexx(nSamp,wRaw,indx);

	  for(l=0;l<nSamp;l++)
	    {
	      j = indx[l+1]-1;
	      max_RS=-1e99;
	      for(k=0;k<nq_Amp;k++)
		{
		  cAmp=(long)((1-upperq[k])*np);
		  nAmp=np-cAmp;

		  if(rankData[i][j]>cAmp && rankData[i][j]!=-1)
		    {
		      RS[k]+=rankData[i][j]-cAmp-0.5*(nAmp+1.0);
		      std[k]+=(nAmp*nAmp-1.0)/12.0;
		    }
		 
		  if(std[k]!=0)
		    {
		      if(RS[k]/sqrt(std[k]) > max_RS)
			max_RS = RS[k]/sqrt(std[k]);
		    }
		  else
		    {
		      if(0 > max_RS)
			max_RS = 0;
		    }
		}
	      lout_RS_Amp[l][kk] += (max_RS-mu_max_Amp_lo[l])/sig_max_Amp_lo[l];
	    }
	}

      
      for(l=0;l<nSamp;l++)
	lout_RS_Amp[l][kk]*=1.0/(double)(endSegAmp[kk]-startSegAmp[kk]+1);

      //compute p-values
      h=endSegAmp[kk]-startSegAmp[kk]+1;
       f=1.0/(double)h;
      for(l=1;l<=h-1;l++)
	f+=2*(double)(h-l)*estAcf[l]/(double)(h*h);
      f=sqrt(f);
      
      for(l=0;l<nSamp;l++)
	{
	  if(lout_RS_Amp[l][kk]>0)
	    lout_p_Amp[l][kk] = erfc(fabs(lout_RS_Amp[l][kk])/(sqrt(2)*f));
	  else
	    lout_p_Amp[l][kk] = 1;
	}
	
    }

  //compute q-values
  for(l=0;l<nSamp;l++)
    qvalues(lout_p_Amp[l],lout_q_Amp[l]);

  //compute leave out rank sums, p, and q values for DELETIONS
  //1) acf computation for p-value determination
  maxL=0;
  for(i=0;i<nSegDel;i++)
    {
      segStart=startSegDel[i];
      segEnd=endSegDel[i];
      //what is the maximal length of the segments?
      if(segEnd-segStart+1 > maxL)
	maxL=segEnd-segStart+1;
    }

  acf(rankSumDel,estAcf,(long)maxL);

  //2) segment averaged leave out statistics
  for(kk=0;kk<nSegDel;kk++)
    {
      for(i=startSegDel[kk];i<=endSegDel[kk];i++)
	{
	  RS = vector<double>(nq_Del,0);
	  std = vector<double>(nq_Del,0);
	  //index rank matrix
	  for(j=0;j<nSamp;j++)
	    wRaw[j+1] = -rankData[i][j];
	  indexx(nSamp,wRaw,indx);

	  for(l=0;l<nSamp;l++)
	    {
	      j = indx[l+1]-1;
	      min_RS=1e99;
	      for(k=0;k<nq_Del;k++)
		{
		  cDel=(long)(lowerq[k]*np);
		  nDel=cDel;
		  
		  if(rankData[i][j]<=cDel && rankData[i][j]!=-1)
		    {
		      RS[k]+=rankData[i][j]-0.5*(nDel+1.0);
		      std[k]+=(nDel*nDel-1.0)/12.0;
		    }
		  
		  if(std[k]!=0)
		    {
		      if(RS[k]/sqrt(std[k])  < min_RS)
			min_RS = RS[k]/sqrt(std[k]);
		    }
		  else
		    {
		      if(0  < min_RS)
			min_RS = 0;
		    }
		}
	      lout_RS_Del[l][kk] += (min_RS-mu_min_Del_lo[l])/sig_min_Del_lo[l];
	    }
	}
      
      
      for(l=0;l<nSamp;l++)
	lout_RS_Del[l][kk]*=1.0/(double)(endSegDel[kk]-startSegDel[kk]+1);

      //compute p-values
      h=endSegDel[kk]-startSegDel[kk]+1;
       f=1.0/(double)h;
      for(l=1;l<=h-1;l++)
	f+=2*(double)(h-l)*estAcf[l]/(double)(h*h);
      f=sqrt(f);
      
      for(l=0;l<nSamp;l++)
	{
	  if(lout_RS_Del[l][kk]<0)
	    lout_p_Del[l][kk] = erfc(fabs(lout_RS_Del[l][kk])/(sqrt(2)*f));
	  else
	    lout_p_Del[l][kk] = 1;
	}
    }

  //compute q-values
  for(l=0;l<nSamp;l++)
    qvalues(lout_p_Del[l],lout_q_Del[l]);


  //leave out output
  for(k=0;k<nSegAmp;k++)
    {
      for(l=0;l<nSamp;l++)
	{
	  if(lout_q_Amp[l][k] <= alfq)
	    break;
	}
      n_leave_out_Amp.push_back(nSamp-l);
      if(l==0)
       	l=1;
      res_RS_Amp.push_back(lout_RS_Amp[l-1][k]);
      res_p_Amp.push_back(lout_p_Amp[l-1][k]);
      res_q_Amp.push_back(lout_q_Amp[l-1][k]);
    }
  
  for(k=0;k<nSegDel;k++)
    {
      for(l=0;l<nSamp;l++)
	{
	  if(lout_q_Del[l][k] <= alfq)
	    break;
	}
      n_leave_out_Del.push_back(nSamp-l);
      if(l==0)
       	l=1;
      res_RS_Del.push_back(lout_RS_Del[l-1][k]);
      res_p_Del.push_back(lout_p_Del[l-1][k]);
      res_q_Del.push_back(lout_q_Del[l-1][k]);
    }


  free_dvector(wRaw,1,nSamp);
  free_lvector(indx,1,nSamp);
  
}

void::mainCLS::writeOutput()
{
  long i,j;
  long segStart,segEnd;
  vector<long> startChr,endChr;
  string tmp,err;
  ofstream out;
    //output
  tmp=outName+"_alf.txt";
  out.open(tmp.c_str());
  startChr.push_back(0);
  for(i=1;i< chromosome.size();i++)
    {
      if(chromosome[i-1]!=chromosome[i])
	{
	  endChr.push_back(i-1);
	  startChr.push_back(i);
	}
    }
  endChr.push_back(chromosome.size()-1);
  
 
  out << segName << endl;
  out << "Id\tType\tCytoband\tChromosome\tStart\tEnd\tSize_Mb\tavgRS\tpValues\tqValues\tn_leave_out\tres_avgRS\tres_pValue\tres_qValue\tGenes\n";

  j=1;
  for(i=0;i<nSegAmp;i++)
    {
      segStart=startSegAmp[i];
      segEnd=endSegAmp[i];
      if(meanRSAmp[i]<0)
	{
	  pAmp[i]=1; qAmp[i]=1;
	}
      out << j << "\t" << "Amp\t" << cyAmp[i] << "\t" << chromosome[segStart] << "\t" << position[segStart] << "\t";
      out << position[segEnd] << "\t" << (position[segEnd]-position[segStart])*1e-6<< "\t"; 
      out << meanRSAmp[i] << "\t" << pAmp[i] <<  "\t" << qAmp[i] << "\t";
      out << n_leave_out_Amp[i] << "\t" << res_RS_Amp[i]  << "\t" << res_p_Amp[i]  << "\t" << res_q_Amp[i]; 
      if(annotAmp[i]=="")
	out << "\t####" << endl;
      else
	out << "\t" << annotAmp[i] << endl;	 
      
      j++;
    }

  
  for(i=0;i<nSegDel;i++)
    {
      segStart=startSegDel[i];
      segEnd=endSegDel[i];
      if(meanRSDel[i]>0)
	{
	  pDel[i]=1; qDel[i]=1;
	}
      
      out << j << "\t" << "Del\t" << cyDel[i] << "\t"<< chromosome[segStart] << "\t" << position[segStart] << "\t";
      out << position[segEnd] << "\t" << (position[segEnd]-position[segStart])*1e-6<< "\t";
      out << meanRSDel[i] << "\t" << pDel[i] <<  "\t" << qDel[i] << "\t";
      out << n_leave_out_Del[i] << "\t" << res_RS_Del[i]  << "\t" << res_p_Del[i]  << "\t" << res_q_Del[i]; 
      if(annotDel[i]=="")
	out << "\t####" << endl;
      else
	out << "\t" << annotDel[i] << endl;	 
      
      j++;
    }
  out.close();
}

void mainCLS::annotateSeg()
{
  ifstream in,inCy;
  string tmp,sep=",",err;
  long i,j,c,sta,sto;

  vector<long> aChr,txStart,txEnd,startChr,endChr;
  vector<long> aChrCy,txStartCy,txEndCy,startChrCy,endChrCy;
  vector<string> gene,cytoband,tmpStr;

  annotAmp.clear();
  annotDel.clear();


  cyAmp.clear();
  cyDel.clear();


  if(build=="hg17")
    {
      tmp=(string)INSTALLDIR;
      tmp+="/annotation/genes_hg17.txt";
      in.open(tmp.c_str());
      tmp=""; tmp=(string)INSTALLDIR;
      tmp+="/annotation/cytoband_hg17.txt";
      inCy.open(tmp.c_str());
    }
  if(build=="hg18")
    {
      tmp=(string)INSTALLDIR;
      tmp+="/annotation/genes_hg18.txt";
      in.open(tmp.c_str());
      tmp=""; tmp=(string)INSTALLDIR;
      tmp+="/annotation/cytoband_hg18.txt";
      inCy.open(tmp.c_str());
    }
  if(build=="mm9")
    {
      tmp=(string)INSTALLDIR;
      tmp+="/annotation/genes_mm9.txt";
      in.open(tmp.c_str());
      tmp=""; tmp=(string)INSTALLDIR;
      tmp+="/annotation/cytoband_mm9.txt";
      inCy.open(tmp.c_str());
    }

  //read gene annotation file
  if(in.is_open())
    {
      //read annotation file
      while(in >> c >> sta >> sto >> tmp)
	{
	  aChr.push_back(c);
	  txStart.push_back(sta);
	  txEnd.push_back(sto);
	  gene.push_back(tmp);
	}

      //find chromosome boundaries
      startChr.push_back(0);
      for(i=1;i< aChr.size();i++)
	{
	  if(aChr[i-1]!=aChr[i])
	    {
	      endChr.push_back(i-1);
	      startChr.push_back(i);
	    }
	}
      endChr.push_back(aChr.size()-1);

    }
  else
    {
      err="Error: cannot find gene annotation file";
      throw err;
    }

  //read cytoband annotation file
  if(inCy.is_open())
    {
      //read annotation file
      while(inCy >> c >> sta >> sto >> tmp)
	{
	  aChrCy.push_back(c);
	  txStartCy.push_back(sta);
	  txEndCy.push_back(sto);
	  cytoband.push_back(tmp);
	}

      //find chromosome boundaries
      startChrCy.push_back(0);
      for(i=1;i< aChrCy.size();i++)
	{
	  if(aChrCy[i-1]!=aChrCy[i])
	    {
	      endChrCy.push_back(i-1);
	      startChrCy.push_back(i);
	    }
	}
      endChrCy.push_back(aChrCy.size()-1);

    }
  else
    {
      err="Error: cannot find cytoband annotation file";
      throw err;
    }


  //AMPLIFICATIONS
  for(i=0;i<startSegAmp.size();i++)
    {
      sta=position[startSegAmp[i]];
      sto=position[endSegAmp[i]];
      tmp="";
      for(j=startChr[atol(chromosome[startSegAmp[i]].c_str())-1];j<=endChr[atol(chromosome[startSegAmp[i]].c_str())-1];j++)
	{
	  if((txStart[j]>=sta && txStart[j] <= sto) || (txEnd[j]>=sta && txEnd[j] <= sto) || (sta >= txStart[j] && sta<= txEnd[j] && sto >= txStart[j] && sto<= txEnd[j]))
	    {
	      tmp+=gene[j]+sep; 
	    }
	}
      annotAmp.push_back(tmp);
      //cytoband
      tmpStr.clear();
      for(j=startChrCy[atol(chromosome[startSegAmp[i]].c_str())-1];j<=endChrCy[atol(chromosome[startSegAmp[i]].c_str())-1];j++)
	{
	  if((txStartCy[j]>=sta && txStartCy[j] <= sto) || (txEndCy[j]>=sta && txEndCy[j] <= sto) || (sta >= txStartCy[j] && sta<= txEndCy[j] && sto >= txStartCy[j] && sto<= txEndCy[j]))
	    {
	      tmpStr.push_back(cytoband[j]); 
	    }
	}
      if(tmpStr.size()>=2)
	tmp=tmpStr[0]+"-"+tmpStr[1];
      else if(tmpStr.size()==1)
	tmp=tmpStr[0];
      else
	tmp="";
      cyAmp.push_back(tmp);
    }

  //DELETIONS
  for(i=0;i<startSegDel.size();i++)
    {
      sta=position[startSegDel[i]];
      sto=position[endSegDel[i]];
      tmp="";
      for(j=startChr[atol(chromosome[startSegDel[i]].c_str())-1];j<=endChr[atol(chromosome[startSegDel[i]].c_str())-1];j++)
	{
	  if((txStart[j]>=sta && txStart[j] <= sto) || (txEnd[j]>=sta && txEnd[j] <= sto) || (sta >= txStart[j] && sta<= txEnd[j] && sto >= txStart[j] && sto<= txEnd[j]))
	    {
	      tmp+=gene[j]+sep; 
	    }
	}
      annotDel.push_back(tmp);
      //cytoband
      tmpStr.clear();
      for(j=startChrCy[atol(chromosome[startSegDel[i]].c_str())-1];j<=endChrCy[atol(chromosome[startSegDel[i]].c_str())-1];j++)
	{
	  if((txStartCy[j]>=sta && txStartCy[j] <= sto) || (txEndCy[j]>=sta && txEndCy[j] <= sto) || (sta >= txStartCy[j] && sta<= txEndCy[j] && sto >= txStartCy[j] && sto<= txEndCy[j]))
	    {
	      tmpStr.push_back(cytoband[j]); 
	    }
	}
      if(tmpStr.size()>=2)
	tmp=tmpStr[0]+"-"+tmpStr[1];
      else if(tmpStr.size()==1)
	tmp=tmpStr[0];
      else
	tmp="";
      cyDel.push_back(tmp);
    }
}

//###################
//## plot module   ##
//###################

void mainCLS::plot(int argc,char *argv[])
{
  int longindex,opt;
  string err;
  double dtmp;
  double alpha=0.05;
  long peel_off = 2;
  ifstream in;
  
  //default parameters/initialize
  build="hg18";

  // option definition
  static struct option longopts[]={
    {"help"  , 0, 0,  'h'},
    {"help"  , 0, 0,  '?'},
    {"b"     , 1, 0,   1},
    {"a"     , 1, 0,   2},
    {"p"     , 1, 0,   3},
    {0, 0, 0, 0}
  }; 

  optind=0;
  //parse command line arguments
  while((opt=getopt_long_only(argc,argv,"h?",longopts,&longindex)) != -1)
    {
      switch(opt)
	{
	case 'h':
	  print_header();
	  cerr << usage_plot << endl;
	  exit(1);
	  break;
	case '?':
	  print_header();
	  cerr << usage_plot << endl;
	  exit(1);	
	  break;
	case 1:
	  build=(string)optarg;
	  if(build!="hg17" && build!="hg18" && build!="mm9")
	    {
	      err="Error: build must either be hg18, hg17, mm9.";
	      throw err;
	    }
	  break;
	case 2:
	  alpha=atof(optarg);
	  break;
	case 3:
	  peel_off = atol(optarg);
	  break;
	default:
	  err="Error: cannot parse arguments.";
	  throw err;
	}
    }

  if((argc-optind)!=2)
    {
      print_header();
      cerr << usage_plot << endl;
      err="Error: please specify name of an alf file.";
      throw err;
    }
  else
    {
      alfName=(string)argv[optind+1];
    }

   //check for alf-file
  in.open((alfName+"_alf.txt").c_str());
  if(!in.is_open())
    {
      err = "Error: cannot open alf-file.";
      throw err;
    }
  in.close();

    //file pointer for interactive R
  FILE *fp;
  string tmp;

  fp=popen("R --slave","w"); // open R
  
  if(fp==NULL)
    {
      cerr << "Error: cannot open R.\n";
      exit(1);
    }

  tmp=(string)INSTALLDIR+"/R/plot_profile.R";
  fprintf(fp,"source(\"%s\")\n",tmp.c_str());
  fprintf(fp,"pdf(\"%s.pdf\",paper=\"a4r\",width=11,height=8)\n",alfName.c_str());

  tmp=(string)INSTALLDIR+"/annotation/cytoband_"+build+".txt";

  if(build!="mm9")
    fprintf(fp,"plot_profile(\"%s_alf.txt\",\"%s\",22,%f,%d)\n",alfName.c_str(),tmp.c_str(),alpha,(int)peel_off);
  else
    fprintf(fp,"plot_profile(\"%s_alf.txt\",\"%s\",19,%f,%d)\n",alfName.c_str(),tmp.c_str(),alpha,(int)peel_off);

  //close R stream
  fprintf(fp,"o=dev.off()\n");
  pclose(fp);

}

//#####################
//## other functions ##
//#####################

void qvalues(vector<double> p,vector<double>&q)
{
  long i,j,n;
  long *ind;
  double *pval,p_pre;
  long n_int,n_ges=0;
  vector<long> n_cum;

  q.clear();

  n=p.size();
  pval=dvector(1,n);
  ind=lvector(1,n);

  for(i=1;i<=n;i++)
    pval[i]=p[i-1];
  
  indexx(n,pval,ind);
  
  p_pre=p[ind[1]-1]; n_int=0;
  for(i=1;i<=n;i++)
    {
      if(p_pre!=p[ind[i]-1])
	{
	  n_ges+=n_int;
	  for(j=0;j<n_int;j++)
	    n_cum.push_back(n_ges);
	  n_int=1; 
	}
      else
	n_int++;

      p_pre=p[ind[i]-1];
    }
  n_ges+=n_int;
  for(j=0;j<n_int;j++)
    n_cum.push_back(n_ges);

  q=vector<double> (n,0);
  for(i=1;i<=n;i++)
    {
      q[ind[i]-1]=p[ind[i]-1]*(double)n/(double)n_cum[i-1];
      if(q[ind[i]-1]>1)
	q[ind[i]-1]=1;
    }
}

void acf(vector<double> dat,vector<double> &acfOut,long k)
{
  long i,j,n=dat.size(),xx;
  double mean=0,var=0;
  long m=20;
  double *x=dvector(1,m);
  double *y=dvector(1,m);
  double *w=dvector(1,m);
  double *g=dvector(1,m);
  double *gam=dvector(1,m);

  acfOut.clear();
  acfOut=vector<double> (k+1,0);

  for(i=0;i<n;i++)
    mean+=dat[i];
  mean*=1.0/(double)n;

  for(i=0;i<=m;i++)
    {
      xx=(long((double)k*(double)i/(double)m));
      acfOut[xx]=0;
      for(j=0;j<n-xx;j++)
	acfOut[xx]+=(dat[j]-mean)*(dat[j+xx]-mean);
      acfOut[xx]*=1.0/(double)(n-xx);
      if(xx==0)
	var=acfOut[0];
      acfOut[xx]*=1.0/var;
      if(i>0)
	{
	  x[i]=xx;
	  y[i]=acfOut[xx];
	  w[i]=1;
	}    
    }
  
  splines(x,y,w,m,0,g,gam);
  gam[m]=0;
  for(i=m-1;i>=2;i--)
    gam[i]=gam[i-1];
  gam[1]=0;

  for(i=1;i<=k;i++)
    {
      acfOut[i]=plot_splines(x,g,gam,m,(double)i);
    }

  free_dvector(x,1,m);
  free_dvector(y,1,m);
  free_dvector(w,1,m);
  free_dvector(g,1,m);
  free_dvector(gam,1,m);
}

void segment(vector<double> dat,vector<long> &segStart,vector<long> &segEnd,double alpha,long minWd)
{
  long i,n,bkp;
  vector<long> tmpStart,tmpEnd;
  list<long> finalBKP;
  double score,maxScore,S1,S2,n1,n2,SQ1,SQ2,v1,v2;
  n=dat.size();
  tmpStart.push_back(0);
  tmpEnd.push_back(n-1);
  finalBKP.push_back(0);
  finalBKP.push_back(n-1);
   
  //clear seg-vectors
  segStart.clear(); segEnd.clear();
  //main loop
  while(tmpStart.size()!=0)
    {
      S1=0; S2=0;
      SQ1=0; SQ2=0;
      n1=0; n2=0;
      maxScore=-1;
      //initialize sub segment
      for(i=tmpStart[0];i<=tmpEnd[0];i++)
	{
	  if(i-tmpStart[0]<minWd-1)
	    {
	      S1+=dat[i];
	      SQ1+=dat[i]*dat[i];
	      n1+=1;
	    }
	  else
	    {
	      S2+=dat[i];
	      SQ2+=dat[i]*dat[i];
	      n2+=1;
	    }
	}
      for(i=0;i<=(tmpEnd[0]-tmpStart[0]+1)-2*minWd;i++)
	{
	  //compute t-statistic for all sub-segments
	  S1+=dat[tmpStart[0]+minWd+i];
	  SQ1+=dat[tmpStart[0]+minWd+i]*dat[tmpStart[0]+minWd+i];
	  S2-=dat[tmpStart[0]+minWd+i];
	  SQ2-=dat[tmpStart[0]+minWd+i]*dat[tmpStart[0]+minWd+i];
	  n1+=1;
	  n2-=1;
	  v1=SQ1/n1-S1*S1/(n1*n1);
	  v2=SQ2/n2-S2*S2/(n2*n2);
	  score=fabs(S1/n1-S2/n2)/sqrt(v1/n1+v2/n2);
	  if(score>maxScore)
	    {
	      maxScore=score;
	      bkp=tmpStart[0]+minWd+i;
	    }
	}
      //is maxScore significantly high?
      if(erfc(maxScore/sqrt(2)) <=alpha && tmpEnd[0]-tmpStart[0]+1 >=minWd)
	{
	  finalBKP.push_back(bkp);
	  //subdivide segment 
	  tmpStart.push_back(tmpStart[0]);
	  tmpEnd.push_back(bkp);
	  tmpStart.push_back(bkp+1);
	  tmpEnd.push_back(tmpEnd[0]);
	}
      tmpStart.erase(tmpStart.begin());
      tmpEnd.erase(tmpEnd.begin());
    }
  //sort break points
  finalBKP.sort();
  //get segments from break points
  list<long>::iterator it;
  for(it=finalBKP.begin();it!=finalBKP.end();it++)
    {
      if(*it!=n-1)
	{
	  if(*it==0)
	    {
	      segStart.push_back(0);
	      it++;
	      segEnd.push_back(*it);
	      it--;
	    }
	  else
	    {
	      segStart.push_back(*it+1);
	      it++;
	      segEnd.push_back(*it);
	      it--;
	    }
	}

    }
}



bool isInSeg(vector<long> chr,vector<long> start,vector<long> end,long tChr,long tPos)
{
  long chrLow,chrUp;
  long up,low;
  
  chrLow=long(lower_bound(chr.begin(),chr.end(),tChr)-chr.begin());
  chrUp =long(upper_bound(chr.begin(),chr.end(),tChr)-chr.begin());
  
  if(chrUp-chrLow <=1)
    return(0);

  low=long(upper_bound(start.begin()+chrLow,start.begin()+chrUp,tPos)-start.begin());
  up= long(lower_bound(end.begin()+chrLow,end.begin()+chrUp,tPos)-end.begin());
  
  if(up-low<0)
    return(1);
  else
    return(0);
      
}

void expRanks(vector<double> expRaw,vector<long> mapExp,long nExp,vector<long> &expRankVec)
{
  long i,k;

  double *sampDat=dvector(1,nExp);
  long *indx=lvector(1,nExp);
  vector<long> mapExpNew;
  
  for(i=0;i<expRaw.size();i++)
    {
      if(mapExp[i]!=-1)
	{
	  mapExpNew.push_back(mapExp[i]);
	  sampDat[i+1]=expRaw[i];
	}
    }
  indexx(nExp,sampDat,indx);
  for(i=1;i<=nExp;i++)
    {
      k=indx[i];
      expRankVec[mapExpNew[k-1]]=i;
    }

  free_dvector(sampDat,1,nExp);
  free_lvector(indx,1,nExp);
}

void sort_cnv(vector<long> &cnvChr,vector<long> &cnvStart,vector<long> &cnvEnd)
{
  vector<long> chr,start,end;
  double *pos;
  long *indx,i,j,n=cnvChr.size();
  
  pos=dvector(1,n);
  indx=lvector(1,n);

  for(i=0;i<n;i++)
    {
      pos[i+1]=1e9*(cnvChr[i]-1)+0.5*(cnvStart[i]+cnvEnd[i]);
    }
  indexx(n,pos,indx);
  
  for(i=1;i<=n;i++)
    {
      j=indx[i]-1;
      chr.push_back(cnvChr[j]);
      start.push_back(cnvStart[j]);
      end.push_back(cnvEnd[j]);
    }
  
  cnvChr=chr;
  cnvStart=start;
  cnvEnd=end;

  free_dvector(pos,1,n);
  free_lvector(indx,1,n);
}
